pub struct Spanned<T> {
pub span: Span,
pub value: T,
pub comments: Vec<Spanned<Comment>>,
}Expand description
A syntax node with source span information attached.
This is the universal wrapper for AST nodes, analogous to:
A.Locatedin elm/compilerNodein stil4m/elm-syntaxSpannedin many Rust parser crates
Each node carries optional leading comments — comments that appeared immediately before this node in source. This enables round-tripping of comments inside expressions (let-in, case-of, etc.).
Fields
span: Spanvalue: Tcomments: Vec<Spanned<Comment>>Leading comments that appeared immediately before this node in source.
Implementations
Sourceimpl<T> Spanned<T>
impl<T> Spanned<T>
Sourcepub fn dummy(value: T) -> Self
pub fn dummy(value: T) -> Self
Create a spanned node with a dummy span (for synthesized/constructed nodes).
Sourcepub fn map<U>(self, f: impl FnOnce(T) -> U) -> Spanned<U>
pub fn map<U>(self, f: impl FnOnce(T) -> U) -> Spanned<U>
Map the inner value, preserving the span and comments.
Sourcepub fn with_comments(self, comments: Vec<Spanned<Comment>>) -> Self
pub fn with_comments(self, comments: Vec<Spanned<Comment>>) -> Self
Attach leading comments to this node.
Sourcepub fn into_inner(self) -> T
pub fn into_inner(self) -> T
Unwrap the spanned node, discarding the span.